import redis
from sqlalchemy import Column, Integer, String, Boolean, Enum, DateTime
from datetime import datetime, timedelta
from enum import Enum as PyEnum
from config.database import Base
import re  # Import the regular expressions module
from settings import REDIS_HOST, REDIS_PORT, REDIS_DB, REDIS_PASSWORD
from sqlalchemy.orm import Session
import bcrypt

# Set up Redis client using settings from settings.py
redis_client = redis.Redis(
    host=REDIS_HOST,
    port=REDIS_PORT,
    db=REDIS_DB,
    password=REDIS_PASSWORD
)

class Status(PyEnum):
    ACTIVE = "active"
    INACTIVE = "inactive"

class SignUpEmailVerification(Base):
    __tablename__ = 'sign_up_email_verification'

    id = Column(Integer, primary_key=True, index=True)
    email = Column(String(255), nullable=False, unique=True)
    is_email_verified = Column(Boolean, default=False)
    otp = Column(String(4), nullable=True)
    token = Column(String(255), nullable=False)
    resend_count = Column(Integer, default=0)
    last_resend_at = Column(DateTime, nullable=True)
    otp_sent_at = Column(DateTime, nullable=True)
    status = Column(Enum(Status), default=Status.ACTIVE, nullable=False)
    created_at = Column(DateTime, default=datetime.utcnow)
    modified_at = Column(DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)

    @classmethod
    def validate_email(cls, email: str):
        """Validates the email format and checks for '+' in the local part."""
        if not email:
            raise ValueError("Email cannot be empty.")

        # Define a stricter regex pattern for email validation
        email_regex = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'

        if not re.match(email_regex, email):
            raise ValueError("Invalid email format.")

        if '+' in email.split('@')[0]:  # Check only the local part
            raise ValueError("Email cannot contain '+' character.")

    def can_resend_otp(self) -> bool:
        """Check if the user can resend OTP based on a limit of 20 resends within 24 hours."""
        now = datetime.utcnow()

        # Check if 5 minutes have passed since the last OTP was sent
        if self.otp_sent_at and (now - self.otp_sent_at) < timedelta(minutes=5):
            return self.resend_count < 20  

        # Reset the resend count if 24 hours have passed since the last resend
        if self.last_resend_at and (now - self.last_resend_at) > timedelta(hours=24):
            self.resend_count = 0

        # Allow resend if the resend count is less than the limit
        return self.resend_count < 20

    def increment_resend_count(self):
        """Increments the resend count and updates the timestamps for the last resend and OTP sent."""
        self.resend_count += 1
        self.last_resend_at = datetime.utcnow()
        self.otp_sent_at = datetime.utcnow()

        # Generate a new token using bcrypt for better security
        new_token = self.generate_new_token(self.email)

        # Update the token in the model
        self.token = new_token

        # Store the new token in Redis with a 15-minute expiration
        redis_client.setex(new_token, timedelta(minutes=15), "active")  # Token expires in 15 minutes

    def verify_otp(self, provided_otp: str, db: Session) -> bool:
        """Verify the OTP and token."""
        try:
            # Check if the token is valid and not expired
            redis_token = redis_client.get(self.token)
            if redis_token is None:
                raise ValueError("Your session has expired. Please try again later or resend the OTP.")

            # Check if the OTP matches
            if self.otp != provided_otp:
                raise ValueError("Invalid OTP.")

            # Update the verification status and timestamps
            self.is_email_verified = True
            self.modified_at = datetime.utcnow()

            # Remove the token from Redis after verification
            redis_client.delete(self.token)

            # Optionally, create a new token and store it in Redis
            new_token = self.generate_new_token(self.email)
            redis_client.setex(new_token, timedelta(minutes=15), "active")  # Expires in 15 minutes
            self.token = new_token

            # Commit changes to the database
            db.commit()

            return True  # Success

        except ValueError as e:
            raise ValueError(str(e))  # Raise any validation error

        except Exception as e:
            raise ValueError(f"An unexpected error occurred: {str(e)}")  # Handle unexpected errors

    def generate_new_token(self, email: str):
        """Generate a new token by hashing the email with bcrypt."""
        # Encode the email as bytes and generate a salt using bcrypt
        salt = bcrypt.gensalt()

        # Hash the email with the salt to generate a unique token
        token = bcrypt.hashpw(email.encode('utf-8'), salt)

        # Return the token as a string
        return token.decode('utf-8')  # Decode to get a readable string

    def __repr__(self):
        return f"<SignUpEmailVerification(id={self.id}, email={self.email}, status={self.status.value}, resend_count={self.resend_count})>"

class SignUpPhoneVerification(Base):
    __tablename__ = 'sign_up_phone_verification'

    id = Column(Integer, primary_key=True, index=True)
    phone_number = Column(String(10), nullable=False, unique=True)
    is_phone_verified = Column(Boolean, default=False)
    otp = Column(String(4), nullable=True)  # Increased OTP length for security
    token = Column(String(255), nullable=False)
    resend_count = Column(Integer, default=0)
    last_resend_at = Column(DateTime, nullable=True)
    otp_sent_at = Column(DateTime, nullable=True)
    status = Column(Enum(Status), default=Status.ACTIVE, nullable=False)
    created_at = Column(DateTime, default=datetime.utcnow)
    modified_at = Column(DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)

    @classmethod
    def validate_phone_number(cls, phone_number: str):
        """Validates the phone number format."""
        if not phone_number:
            raise ValueError("Phone number cannot be empty.")

        # Validate phone number using regex (simple example for a 10-digit phone number)
        phone_regex = r'^\+?1?\d{9,10}$'

        if not re.match(phone_regex, phone_number):
            raise ValueError("Invalid phone number format. Ensure it's a valid international number.")

    def can_resend_otp(self) -> bool:
        """Check if the user can resend OTP based on a limit of 20 resends within 24 hours."""
        now = datetime.utcnow()

        # Check if 5 minutes have passed since the last OTP was sent
        if self.otp_sent_at and (now - self.otp_sent_at) < timedelta(minutes=5):
            return self.resend_count < 20  

        # Reset the resend count if 24 hours have passed since the last resend
        if self.last_resend_at and (now - self.last_resend_at) > timedelta(hours=24):
            self.resend_count = 0

        # Allow resend if the resend count is less than the limit
        return self.resend_count < 20

    def increment_resend_count(self):
        """Increments the resend count and updates the timestamps for the last resend and OTP sent."""
        self.resend_count += 1
        self.last_resend_at = datetime.utcnow()
        self.otp_sent_at = datetime.utcnow()

        # Generate a new token using bcrypt for better security
        new_token = self.generate_new_token(self.phone_number)

        # Update the token in the model
        self.token = new_token

        # Store the new token in Redis with a 15-minute expiration
        redis_client.setex(new_token, timedelta(minutes=15), "active")  # Token expires in 15 minutes

    def verify_otp(self, provided_otp: str, db: Session) -> bool:
        """Verify the OTP and token."""
        try:
            # Check if the token is valid and not expired
            redis_token = redis_client.get(self.token)
            if redis_token is None:
                raise ValueError("Your session has expired. Please try again later or resend the OTP.")

            # Check if the OTP matches
            if self.otp != provided_otp:
                raise ValueError("Invalid OTP.")

            # Update the verification status and timestamps
            self.is_phone_verified = True
            self.modified_at = datetime.utcnow()

            # Remove the token from Redis after verification
            redis_client.delete(self.token)

            # Optionally, create a new token and store it in Redis
            new_token = self.generate_new_token(self.phone_number)
            redis_client.setex(new_token, timedelta(minutes=15), "active")  # Expires in 15 minutes
            self.token = new_token

            # Commit changes to the database
            db.commit()

            return True  # Success

        except ValueError as e:
            raise ValueError(str(e))  # Raise any validation error

        except Exception as e:
            raise ValueError(f"An unexpected error occurred: {str(e)}")  # Handle unexpected errors

    def generate_new_token(self, phone_number: str):
        """Generate a new token by hashing the phone number with bcrypt."""
        # Encode the phone number as bytes and generate a salt using bcrypt
        salt = bcrypt.gensalt()

        # Hash the phone number with the salt to generate a unique token
        token = bcrypt.hashpw(phone_number.encode('utf-8'), salt)

        # Return the token as a string
        return token.decode('utf-8')  # Decode to get a readable string

    def __repr__(self):
        return f"<SignUpPhoneVerification(id={self.id}, phone_number={self.phone_number}, status={self.status.value}, resend_count={self.resend_count})>"
