import os
from aiohttp import web
from sqlalchemy.orm import Session
from config.database import get_db
from utils.response import format_response
import bcrypt
from datetime import datetime, timedelta
from settings import TRANSMIT_DATA, REDIS_HOST, REDIS_PORT, REDIS_DB, REDIS_PASSWORD
from apps.auth.models import SignUpEmailVerification, Status, SignUpPhoneVerification
import random
from dotenv import load_dotenv
from celery import Celery
import re
import redis
# from twilio.rest import Client

# Load environment variables from the .env file
load_dotenv()

# # Create a client for accessing tasks on the shared Redis broker
# app = Celery(
#     'client', 
#     broker=f'redis://{REDIS_PASSWORD}@{REDIS_HOST}:{REDIS_PORT}/{REDIS_DB}',
#     backend=f'redis://{REDIS_PASSWORD}@{REDIS_HOST}:{REDIS_PORT}/{REDIS_DB}',
#     task_ignore_result=False,
#     result_expires=3600,  # Results available for 1 hour (3600 seconds)
# )

# # Set up Redis client using settings from settings.py
# redis_client = redis.Redis(
#     host=REDIS_HOST,
#     port=REDIS_PORT,
#     db=REDIS_DB,
#     password=REDIS_PASSWORD
# )

# Check if the Redis password is provided
redis_url = f'redis://{REDIS_HOST}:{REDIS_PORT}/{REDIS_DB}'

if REDIS_PASSWORD:
    redis_url = f'redis://{REDIS_PASSWORD}@{REDIS_HOST}:{REDIS_PORT}/{REDIS_DB}'

# Create a client for accessing tasks on the shared Redis broker (Celery setup)
app = Celery(
    'client', 
    broker=redis_url,  # Use the constructed Redis URL
    backend=redis_url,  # Use the same URL for the backend
    task_ignore_result=False,
    result_expires=3600,  # Results available for 1 hour (3600 seconds)
)

# Set up Redis client using settings from settings.py (with or without password)
redis_client = redis.Redis(
    host=REDIS_HOST,
    port=REDIS_PORT,
    db=REDIS_DB,
    password=REDIS_PASSWORD if REDIS_PASSWORD else None  # Include password only if provided
)


# Twilio client setup for SMS sending
# twilio_client = Client(TWILIO_ACCOUNT_SID, TWILIO_AUTH_TOKEN)

async def send_otp(request):
    """Handles sending an OTP to the user's email for account verification."""
    db: Session = next(get_db())
    response_data = TRANSMIT_DATA.copy()

    try:
        data = await request.json()
        email = data.get('email')

        # Validate email presence
        if not email:
            response_data.update({
                "message": "Email is required.",
                "errorList": ["Email is required."],
            })
            return web.json_response(response_data, status=400)

        #  Validate email format
        try:
            SignUpEmailVerification.validate_email(email)
        except ValueError as e:
            response_data.update({
                "message": str(e),
                "errorList": [str(e)],
            })
            return web.json_response(response_data, status=400)    

        # Fetch or create SignUpEmailVerification record
        sign_up_email_verification = db.query(SignUpEmailVerification).filter(SignUpEmailVerification.email == email).first()

        # Handle existing verification record
        if sign_up_email_verification:
            if sign_up_email_verification.is_email_verified:
                response_data.update({
                    "message": "Email is already verified.",
                    "errorList": ["Email is already verified."],
                })
                return web.json_response(response_data, status=400)

            # Check the resend count and OTP resend rules
            if sign_up_email_verification.resend_count >= 20:
                response_data.update({
                    "message": "OTP resend limit reached. Cannot resend OTP.",
                    "errorList": ["OTP resend limit reached."],
                })
                return web.json_response(response_data, status=429)

            # Allow resend of existing OTP if within 5 minutes
            if (datetime.utcnow() - sign_up_email_verification.otp_sent_at) < timedelta(minutes=5):
                otp = sign_up_email_verification.otp
                sign_up_email_verification.increment_resend_count()
                sign_up_email_verification.otp_sent_at = datetime.utcnow()  # Update the time of OTP sent
                response_data.update({
                    "isValid": True,
                    "status": "SUCCESS",
                    "message": "Same OTP sent successfully.",
                })
            else:
                # Generate a new OTP and reset the resend count
                otp = generate_otp()
                sign_up_email_verification.otp = otp
                sign_up_email_verification.otp_sent_at = datetime.utcnow()  # Update the time of OTP sent
                sign_up_email_verification.resend_count = 1  # Reset resend count for new OTP
                response_data.update({
                    "isValid": True,
                    "status": "SUCCESS",
                    "message": "New OTP sent successfully.",
                })

            # Send OTP email using Celery task
            app.send_task("email_server.send_email_task", args=[email, "otp", {"otp": otp}])
            db.commit()
            return web.json_response(response_data)

        # Create new verification record if none exists
        otp = generate_otp()
        token = bcrypt.hashpw(email.encode('utf-8'), bcrypt.gensalt()).decode('utf-8')
        new_sign_up_email_verification = SignUpEmailVerification(
            email=email,
            otp=otp,
            token=token,
            is_email_verified=False,
            resend_count=1,
            otp_sent_at=datetime.utcnow(),
            status=Status.ACTIVE,
            created_at=datetime.utcnow(),
            modified_at=datetime.utcnow()
        )

        db.add(new_sign_up_email_verification)
        db.commit()

        # Set the token in Redis after committing the new record
        redis_client.setex(token, timedelta(minutes=15), "active")  # Token expires in 15 minutes

        # Send OTP email using Celery task
        app.send_task("email_server.send_email_task", args=[email, "otp", {"otp": otp}])
        response_data.update({
            "isValid": True,
            "status": "SUCCESS",
            "message": "New OTP sent successfully."
        })
        return web.json_response(response_data)

    except Exception as e:
        print(f"Error during OTP sending: {e}")
        response_data.update({
            "message": "Failed to send OTP.",
            "errorList": [str(e)],
        })
        return web.json_response(response_data, status=500)

    finally:
        # Ensure the database session is always closed
        db.close()

def generate_otp() -> str:
    """Generate a 4-digit OTP."""
    return str(random.randint(1000, 9999))

async def verify_otp(request):
    """Handles OTP verification for user account activation."""
    db: Session = next(get_db())
    response_data = TRANSMIT_DATA.copy()

    try:
        # Parse JSON input from the request
        data = await request.json()
        email = data.get('email')
        otp = data.get('otp')

        # Validate the email and OTP
        if not email or not otp:
            response_data.update({
                "message": "Email and OTP are required.",
                "errorList": ["Email and OTP are required."]
            })
            return web.json_response(response_data, status=400)

        # Retrieve the SignUpEmailVerification record from the database
        sign_up_email_verification = db.query(SignUpEmailVerification).filter(SignUpEmailVerification.email == email).first()

        if not sign_up_email_verification:
            response_data.update({
                "message": "Verification record not found.",
                "errorList": ["Verification record not found."]
            })
            return web.json_response(response_data, status=404)

        # Call the model method to verify the OTP
        if sign_up_email_verification.verify_otp(otp, db):
            response_data.update({
                "isValid": True,
                "status": "SUCCESS",
                "message": "OTP verified successfully. Your email is now verified."
            })
            return web.json_response(response_data)

    except ValueError as e:
        response_data.update({
            "message": str(e),
            "errorList": [str(e)]
        })
        return web.json_response(response_data, status=400)

    except Exception as e:
        print(f"Error during OTP verification: {e}")
        response_data.update({
            "message": "Failed to verify OTP.",
            "errorList": [str(e)],
        })
        return web.json_response(response_data, status=500)

    finally:
        # Ensure the database session is always closed
        db.close()


async def send_sms_otp(request):
    """Handles sending an OTP to the user's phone number via SMS for account verification."""
    db: Session = next(get_db())
    response_data = TRANSMIT_DATA.copy()

    try:
        data = await request.json()
        phone_number = data.get('phone_number')

        # Validate phone number presence
        if not phone_number:
            response_data.update({
                "message": "Phone number is required.",
                "errorList": ["Phone number is required."],
            })
            return web.json_response(response_data, status=400)

        # Validate phone number format (basic validation, could be enhanced)
        # Check if the phone number is valid
        if not re.match(r'^[1-9][0-9]{9}$', phone_number):
            response_data.update({
                "message": "Invalid phone number format. The phone number must be exactly 10 digits and contain no special characters.",
                "errorList": ["Invalid phone number format. It must be a 10-digit number without any special characters or spaces."],
            })
            return web.json_response(response_data, status=400)

        # Fetch or create SignUpPhoneVerification record
        sign_up_phone_verification = db.query(SignUpPhoneVerification).filter(SignUpPhoneVerification.phone_number == phone_number).first()

        # Handle existing verification record
        if sign_up_phone_verification:
            if sign_up_phone_verification.is_phone_verified:
                response_data.update({
                    "message": "Phone number is already verified.",
                    "errorList": ["Phone number is already verified."],
                })
                return web.json_response(response_data, status=400)

            # Check the resend count and OTP resend rules
            if sign_up_phone_verification.resend_count >= 20:
                response_data.update({
                    "message": "OTP resend limit reached. Cannot resend OTP.",
                    "errorList": ["OTP resend limit reached."],
                })
                return web.json_response(response_data, status=429)

            # Allow resend of existing OTP if within 5 minutes
            if (datetime.utcnow() - sign_up_phone_verification.otp_sent_at) < timedelta(minutes=5):
                otp = sign_up_phone_verification.otp
                sign_up_phone_verification.increment_resend_count()
                sign_up_phone_verification.otp_sent_at = datetime.utcnow()  # Update the time of OTP sent
                response_data.update({
                    "isValid": True,
                    "status": "SUCCESS",
                    "message": "Same OTP sent successfully.",
                })
            else:
                # Generate a new OTP and reset the resend count
                otp = generate_otp()
                sign_up_phone_verification.otp = otp
                sign_up_phone_verification.otp_sent_at = datetime.utcnow()  # Update the time of OTP sent
                sign_up_phone_verification.resend_count = 1  # Reset resend count for new OTP
                response_data.update({
                    "isValid": True,
                    "status": "SUCCESS",
                    "message": "New OTP sent successfully.",
                })

            # Send OTP via SMS using Twilio
            # twilio_client.messages.create(
            #     body=f"Your OTP code is {otp}",
            #     from_=TWILIO_PHONE_NUMBER,
            #     to=phone_number
            # )

            db.commit()
            return web.json_response(response_data)

        # Create new verification record if none exists
        otp = generate_otp()
        token = bcrypt.hashpw(phone_number.encode('utf-8'), bcrypt.gensalt()).decode('utf-8')
        new_sign_up_phone_verification = SignUpPhoneVerification(
            phone_number=phone_number,
            otp=otp,
            token=token,
            is_phone_verified=False,
            resend_count=1,
            otp_sent_at=datetime.utcnow(),
            status=Status.ACTIVE,
            created_at=datetime.utcnow(),
            modified_at=datetime.utcnow()
        )

        db.add(new_sign_up_phone_verification)
        db.commit()

        # Set the token in Redis after committing the new record
        redis_client.setex(token, timedelta(minutes=15), "active")  # Token expires in 15 minutes

        # Send OTP via SMS using Twilio
        # twilio_client.messages.create(
        #     body=f"Your OTP code is {otp}",
        #     from_=TWILIO_PHONE_NUMBER,
        #     to=phone_number
        # )

        response_data.update({
            "isValid": True,
            "status": "SUCCESS",
            "message": "New OTP sent successfully."
        })
        return web.json_response(response_data)

    except Exception as e:
        print(f"Error during OTP sending: {e}")
        response_data.update({
            "message": "Failed to send OTP.",
            "errorList": [str(e)],
        })
        return web.json_response(response_data, status=500)

    finally:
        # Ensure the database session is always closed
        db.close()


async def verify_sms_otp(request):
    """Handles OTP verification for phone number activation."""
    db: Session = next(get_db())
    response_data = TRANSMIT_DATA.copy()

    try:
        # Parse JSON input from the request
        data = await request.json()
        phone_number = data.get('phone_number')
        otp = data.get('otp')

        # Validate the phone number and OTP
        if not phone_number or not otp:
            response_data.update({
                "message": "Phone number and OTP are required.",
                "errorList": ["Phone number and OTP are required."]
            })
            return web.json_response(response_data, status=400)

        # Retrieve the SignUpPhoneVerification record from the database
        sign_up_phone_verification = db.query(SignUpPhoneVerification).filter(SignUpPhoneVerification.phone_number == phone_number).first()

        if not sign_up_phone_verification:
            response_data.update({
                "message": "Verification record not found.",
                "errorList": ["Verification record not found."]
            })
            return web.json_response(response_data, status=404)

        # Call the model method to verify the OTP
        if sign_up_phone_verification.verify_otp(otp, db):
            response_data.update({
                "isValid": True,
                "status": "SUCCESS",
                "message": "OTP verified successfully. Your phone number is now verified."
            })
            return web.json_response(response_data)

    except ValueError as e:
        response_data.update({
            "message": str(e),
            "errorList": [str(e)]
        })
        return web.json_response(response_data, status=400)

    except Exception as e:
        print(f"Error during OTP verification: {e}")
        response_data.update({
            "message": "Failed to verify OTP.",
            "errorList": [str(e)],
        })
        return web.json_response(response_data, status=500)

    finally:
        # Ensure the database session is always closed
        db.close()