import logging
import os
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from celery import shared_task
from jinja2 import Environment, FileSystemLoader, TemplateNotFound
from sqlalchemy.orm import Session

from config.database import get_db
from settings import SMTP_SERVER, SMTP_PORT, SMTP_USERNAME, SMTP_PASSWORD
from utils.redis import redis_client

from .celery import app
from apps.auth.models import SignUpEmailVerification, SignUpPhoneVerification

logger = logging.getLogger(__name__)

# Set up Jinja2 environment to load templates from the email_templates directory
TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), 'email_templates')
env = Environment(loader=FileSystemLoader(TEMPLATE_DIR))

@app.task(bind=True, name="email_server.send_email_task", max_retries=3)
def send_email_task(self, recipients, email_type, context_data=None, bulk=False):
    """
    Celery task to send either a welcome or OTP email to single or multiple recipients.

    Parameters:
    - recipients (list or str): Recipient(s) email address(es).
    - email_type (str): Type of email to send ("welcome" or "otp").
    - context_data (dict): Data for formatting the email content.
    - bulk (bool): If True, send one bulk email with all recipients in the "To" field; otherwise, send individually.
    """

    if isinstance(recipients, str):
        recipients = [recipients]  # Convert single email to a list

    # Select template based on email_type
    subject = ""
    template = None

    try:
        if email_type == "welcome":
            subject = "Welcome to Our Service!"
            template = env.get_template("welcome_email.html")
        elif email_type == "otp":
            subject = "Your OTP Code"
            template = env.get_template("otp_email.html")
        else:
            logger.error(f"Invalid email_type: {email_type}")
            return {"status": "error", "message": "Invalid email_type specified"}

        # Render the email body with the context data
        html_body = template.render(context_data or {})

    except TemplateNotFound as e:
        logger.error(f"Template not found: {e}")
        return {"status": "error", "message": f"Template '{e.name}' not found."}
    except Exception as e:
        logger.error(f"Error rendering template: {e}")
        return {"status": "error", "message": str(e)}

    # Prepare emails based on the bulk parameter
    if bulk:
        # Send one email with multiple recipients in the "To" field
        send_single_email(self, recipients, subject, html_body)
    else:
        # Send individual emails to each recipient
        for recipient in recipients:
            send_single_email(self, [recipient], subject, html_body)

    return {"status": "success", "message": f"{email_type.capitalize()} email sent to {len(recipients)} recipient(s)"}

def send_single_email(task_instance, to_emails, subject, html_body):
    """
    Helper function to send an email with the given recipients, subject, and HTML body.
    """
    try:
        msg = MIMEMultipart("alternative")
        msg['From'] = SMTP_USERNAME
        msg['To'] = ", ".join(to_emails)
        msg['Subject'] = subject

        # Attach the HTML body to the email
        msg.attach(MIMEText(html_body, 'html'))

        # Establish a secure session with SMTP server
        with smtplib.SMTP(SMTP_SERVER, SMTP_PORT) as server:
            server.starttls()
            server.login(SMTP_USERNAME, SMTP_PASSWORD)
            server.send_message(msg)

        logger.info(f"Email sent to: {', '.join(to_emails)}")
    except smtplib.SMTPException as smtp_err:
        logger.error(f"SMTP error occurred while sending email to {', '.join(to_emails)}: {smtp_err}")
        raise task_instance.retry(exc=smtp_err, countdown=60)
    except Exception as e:
        logger.error(f"Failed to send email to {', '.join(to_emails)}: {e}")
        return {"status": "error", "message": str(e)}

# @app.task(bind=True, name="email_server.validate_email_otp_token_task", max_retries=3)
# def validate_email_otp_token_task(self, email: str, phone_number: str,):
#     """
#     Celery task to validate the email, OTP, and token for signup process.

#     Parameters:
#     - email (str): The email address to validate.
#     """
#     response_data = {"status": "ERROR", "message": "", "errorList": []}
#     db: Session = next(get_db())

#     try:
#         # Step 1: Check if email exists in SignUpEmailVerification table
#         sign_up_email_verification = db.query(SignUpEmailVerification).filter(SignUpEmailVerification.email == email).first()
#         if not sign_up_email_verification:
#             response_data["message"] = (
#                 "Your email verification process has not been initiated. Please try again."
#             )
#             logger.warning(response_data["message"])
#             return response_data

#         # Step 2: Check if the token is valid in Redis
#         token = sign_up_email_verification.token
#         redis_status = redis_client.get(token)  # Fetch token status from Redis
#         if redis_status is None:
#             # Token has expired, set is_email_verified to False
#             sign_up_email_verification.is_email_verified = False
#             db.commit()  # Commit the changes to the database
#             response_data["message"] = (
#                 "Your session has expired. Please request a new OTP and try again."
#             )
#             logger.warning(response_data["message"])
#             return response_data

#         # Step 3: Check if OTP is valid
#         # if sign_up_email_verification.otp != otp:
#         #     response_data["message"] = (
#         #         "OTP verification has failed. Please check your OTP and try again."
#         #     )
#         #     logger.warning(response_data["message"])
#         #     return response_data

#         # Step 4: Check if email is verified
#         if not sign_up_email_verification.is_email_verified:
#             response_data["message"] = (
#                 "Your email address has not been verified yet. Please try again."
#             )
#             logger.warning(response_data["message"])
#             return response_data

#         # All validations passed
#         response_data["status"] = "SUCCESS"
#         response_data["message"] = (
#             "All checks are valid! You may proceed with the signup process."
#         )
#         logger.info(response_data["message"])
#         return response_data

#     except Exception as e:
#         response_data["message"] = f"An unexpected error occurred while validating: {str(e)}"
#         logger.error(response_data["message"])
#         return response_data

#     finally:
#         db.close()


@app.task(bind=True, name="email_server.validate_email_otp_token_task", max_retries=3)
def validate_email_otp_token_task(self, email: str, phone_number: str):
    """
    Celery task to validate the email, OTP, token for signup process, and phone number for verification.

    Parameters:
    - email (str): The email address to validate.
    - phone_number (str): The phone number to validate.
    """
    response_data = {"status": "ERROR", "message": "", "errorList": []}
    db: Session = next(get_db())

    try:
        # Step 1: Check if email exists in SignUpEmailVerification table
        sign_up_email_verification = db.query(SignUpEmailVerification).filter(SignUpEmailVerification.email == email).first()
        if not sign_up_email_verification:
            response_data["message"] = (
                "Your email verification process has not been initiated. Please try again."
            )
            logger.warning(response_data["message"])
            return response_data

        # Step 2: Check if the token is valid in Redis
        token = sign_up_email_verification.token
        redis_status = redis_client.get(token)  # Fetch token status from Redis
        if redis_status is None:
            # Token has expired, set is_email_verified to False
            sign_up_email_verification.is_email_verified = False
            db.commit()  # Commit the changes to the database
            response_data["message"] = (
                "Your session has expired. Please request a new OTP and try again."
            )
            logger.warning(response_data["message"])
            return response_data

        # Step 3: Check if OTP is valid (Uncomment and adjust if necessary)
        # if sign_up_email_verification.otp != otp:
        #     response_data["message"] = (
        #         "OTP verification has failed. Please check your OTP and try again."
        #     )
        #     logger.warning(response_data["message"])
        #     return response_data

        # Step 4: Check if email is verified
        if not sign_up_email_verification.is_email_verified:
            response_data["message"] = (
                "Your email address has not been verified yet. Please try again."
            )
            logger.warning(response_data["message"])
            return response_data

        # Step 5: Check if phone number exists in SignUpPhoneVerification table
        sign_up_phone_verification = db.query(SignUpPhoneVerification).filter(SignUpPhoneVerification.phone_number == phone_number).first()
        if not sign_up_phone_verification:
            response_data["message"] = (
                "Your phone number verification process has not been initiated. Please try again."
            )
            logger.warning(response_data["message"])
            return response_data

        # Step 6: Check if phone number is verified
        if not sign_up_phone_verification.is_phone_verified:
            response_data["message"] = (
                "Your phone number has not been verified yet. Please try again."
            )
            logger.warning(response_data["message"])
            return response_data

        # Step 7: Check if the phone number token is valid in Redis
        phone_token = sign_up_phone_verification.token
        phone_redis_status = redis_client.get(phone_token)  # Fetch phone token status from Redis
        if phone_redis_status is None:
            # Token has expired, set is_phone_verified to False
            sign_up_phone_verification.is_phone_verified = False
            db.commit()  # Commit the changes to the database
            response_data["message"] = (
                "Your phone number verification session has expired. Please request a new OTP and try again."
            )
            logger.warning(response_data["message"])
            return response_data

        # All validations passed for both email and phone number
        response_data["status"] = "SUCCESS"
        response_data["message"] = (
            "All checks are valid! You may proceed with the signup process."
        )
        logger.info(response_data["message"])
        return response_data

    except Exception as e:
        response_data["message"] = f"An unexpected error occurred while validating: {str(e)}"
        logger.error(response_data["message"])
        return response_data

    finally:
        db.close()

